import { dirname, extname } from 'path';
import { existsSync } from 'fs';
const UNEXPECTED_ERROR_LOCATION = { line: 1, column: 0 };
export function defineRule(ruleName, rule) {
    return {
        meta: {
            ...rule.meta,
            docs: {
                ...rule.meta.docs,
                url: 'https://github.com/intlify/eslint-plugin-svelte/blob/main/docs/rules/' +
                    ruleName +
                    '.md'
            }
        },
        create(context) {
            const sourceCode = context.sourceCode;
            if (!sourceCode.parserServices.isSvelte) {
                const filename = context.filename;
                if (isSvelteFile(filename)) {
                    context.report({
                        loc: UNEXPECTED_ERROR_LOCATION,
                        message: 'Use the latest svelte-eslint-parser. See also https://github.com/intlify/eslint-plugin-svelte/blob/main/docs/started.md#faq'
                    });
                }
                return {};
            }
            return rule.create(context);
        }
    };
}
function isSvelteFile(filename) {
    if (!filename.includes('.svelte')) {
        return false;
    }
    let target = filename;
    while (!existsSync(target)) {
        const next = dirname(target);
        if (!next || next === target) {
            return false;
        }
        target = next;
    }
    return extname(target) === '.svelte';
}
